#!/bin/bash

#    Automates the packaging process of the software "xf86-video-nvidia" for
#    use with Zenwalk Linux.
#    
#    Copyright (C) 2008 Michael Verret
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#    ${SOFTNAME} Home Page: http://www.nvidia.com/
#    ${SOFTNAME} Sources: http://us.download.nvidia.com/XFree86/Linux-x86/177.82/NVIDIA-Linux-x86-180.22-pkg1.run
#    ${SOFTNAME} Dependencies: "atk,expat,fontconfig,freetype,gcc,glib2,gtk+2,libpng,pango,xorg-libs,xcb,zlib"


# Define variables


SOURCEVERSION="260.19.29"
KERNELVERSION="$(uname -r)"
MODULESVERSION="$(uname -r)"
BUILDIR=$(pwd)
SOURCEDIR="NVIDIA-Linux-x86-${SOURCEVERSION}"
SOFTNAME="xf86-video-nvidia"
PACKAGE="${SOFTNAME}-${SOURCEVERSION}-i486-${KERNELVERSION}"
INSTALLDIR=/tmp/${PACKAGE}
SOURCES="NVIDIA-Linux-x86-${SOURCEVERSION}.run"

# Make Sure That We Have Everything


for file in ${SOURCES} ; do
  echo -n "$file :"
  if [ -e $file ]; then
    echo " OK"
  else
    echo " MISSING !"
    exit 1
  fi
done


# Ensure clean work area


rm -rf ${INSTALLDIR} ${SOURCEDIR} ${PACKAGE}*


# Extract the source


chmod +x NVIDIA-Linux-x86-${SOURCEVERSION}-pkg.run

sh ${SOURCEDIR}.run --extract-only


# Ensure sane permissions


cd ${BUILDIR}/${SOURCEDIR}/
	chown -R root:root .
    find . -perm 666 -exec chmod 644 {} \;
    find . -perm 664 -exec chmod 644 {} \;
    find . -perm 600 -exec chmod 644 {} \;
    find . -perm 444 -exec chmod 644 {} \;
    find . -perm 400 -exec chmod 644 {} \;
    find . -perm 440 -exec chmod 644 {} \;
    find . -perm 777 -exec chmod 755 {} \;
    find . -perm 775 -exec chmod 755 {} \;
    find . -perm 511 -exec chmod 755 {} \;
    find . -perm 711 -exec chmod 755 {} \;
    find . -perm 555 -exec chmod 755 {} \;
    
    
# Install the NVIDIA drivers utilities and libraries


cd ${BUILDIR}/${SOURCEDIR}

mkdir -p ${INSTALLDIR}/usr/{lib,bin,share/applications,share/icons/hicolor/48x48/apps}
mkdir -p ${INSTALLDIR}/usr/lib/xorg/modules/{extensions,drivers}

install ./{libGLcore,libGL,libnvidia-cfg,libnvidia-glcore,libcuda,libvdpau,libvdpau_nvidia,tls/libnvidia-tls}.so.${SOURCEVERSION} \
       ${INSTALLDIR}/usr/lib/ || return 1

install ./libXv* ${INSTALLDIR}/usr/lib/ || return 1

install ./libnvidia-wfb.so.${SOURCEVERSION} \
	${INSTALLDIR}/usr/lib/xorg/modules || return 1

install ./nvidia_drv.so \
	${INSTALLDIR}/usr/lib/xorg/modules/drivers || return 1

install ./libglx.so.${SOURCEVERSION} \
	${INSTALLDIR}/usr/lib/xorg/modules/extensions || return 1

( cd ${BUILDIR}/${SOURCEDIR}/usr/bin/
	install -m755 nvidia-settings nvidia-xconfig nvidia-bug-report.sh \
	${INSTALLDIR}/usr/bin/ || return 1
)

( cd ${INSTALLDIR}/usr/lib/
for a in ./*.so.${SOURCEVERSION}* ; do ln -sf $a  ${a%%.so.*}.so ; done
for a in ./*.so.${SOURCEVERSION}* ; do ln -sf $a  ${a%%.so.*}.so.1 ; done
)

( cd ${INSTALLDIR}/usr/lib/xorg/modules/extensions
	ln -s libglx.so.${SOURCEVERSION} libglx.so || return 1
)


# Install the nvidia .desktop file


install -m644 ./nvidia-settings.desktop ${INSTALLDIR}/usr/share/applications/ || return 1

sed -e 's:__UTILS_PATH__:/usr/bin:' -e 's:Icon=__PIXMAP_PATH__/:Icon=:' -i \
	${INSTALLDIR}/usr/share/applications/nvidia-settings.desktop

install -m644 ./nvidia-settings.png \
	${INSTALLDIR}/usr/share/icons/hicolor/48x48/apps/ || return 1

( cd ${INSTALLDIR}/usr/share/licenses/ ; ln -s nvidia nvidia-utils || return 1
)


# Install the nvidia documentation


( cd ${BUILDIR}/${SOURCEDIR}
	install -D -m644 ./README.txt \
	${INSTALLDIR}/usr/doc/${SOFTNAME}-${SOURCEVERSION}./README || return 1
)

( cd ${BUILDIR}/${SOURCEDIR}
	install -m644 LICENSE ${INSTALLDIR}/usr/doc/${SOFTNAME}-${SOURCEVERSION}./ || return 1
)

  
find ${BUILDIR}/${SOURCEDIR}/usr -type d -exec chmod 755 {} \;


# Build the nvidia kernel module


( cd ${BUILDIR}/${SOURCEDIR}/kernel/
	make
	make -f Makefile.kbuild module
)


# Install the nvidia kernel module


( cd ${BUILDIR}/${SOURCEDIR}/kernel/
	mkdir -p ${INSTALLDIR}/lib/modules/${MODULESVERSION}/kernel/drivers/video/
	install -m644 nvidia.ko ${INSTALLDIR}/lib/modules/${MODULESVERSION}/kernel/drivers/video/
)


# Post-install routine


mkdir -p ${INSTALLDIR}/install
cat << "EOF" > ${INSTALLDIR}/install/doinst.sh
#!/bin/sh
depmod `uname -r`
rmmod nvidia  > /dev/null 2>&1
modprobe nvidia > /dev/null 2>&1

# Replace nv driver with nvidia when doing install.
if [[ -f /etc/X11/xorg.conf ]]; then
	sed -i '/^[[:space:]]*Driver[[:space:]]*"nv"/s/nv/nvidia/' /etc/X11/xorg.conf
	sed -i '/^[[:space:]]*Load[[:space:]]*".*glx.*"/s|".*"|"/usr/lib/xorg/modules/extensions/libglx.so"|' /etc/X11/xorg.conf

# Enable hardware acceleration of the RENDER extension
	grep -q '^[[:space:]]*Option "RenderAccel"' /etc/X11/xorg.conf || sed -i '/^[[:space:]]*Driver[[:space:]]*"nvidia".*$/s/$/\n    Option "RenderAccel" "1"/' /etc/X11/xorg.conf

# Enable GLX even when the Composite X extension is loaded
	grep -q '^[[:space:]]*Option "AllowGLXWithComposite"' /etc/X11/xorg.conf || sed -i '/^[[:space:]]*Driver[[:space:]]*"nvidia".*$/s/$/\n    Option "AllowGLXWithComposite" "1"/' /etc/X11/xorg.conf
fi
EOF


# Add Slack-desc


mkdir -p ${INSTALLDIR}/install
cat <<E1F > ${INSTALLDIR}/install/slack-desc 
xf86-video-nvidia: xf86-video-nvidia (NVIDIA Accelerated Linux Graphics Driver)
xf86-video-nvidia: 
xf86-video-nvidia: The NVIDIA Accelerated Linux Graphics Driver brings
xf86-video-nvidia: accelerated 2D functionality and high-performance
xf86-video-nvidia: OpenGL support to Linux x86 with the use of NVIDIA
xf86-video-nvidia: graphics processing units (GPUs).
xf86-video-nvidia: This is a version for the Real Time kernel.
xf86-video-nvidia: 
xf86-video-nvidia: Build Details:
xf86-video-nvidia: Linux Kernel Version - linux-${KERNELVERSION}
xf86-video-nvidia: NVIDIA Driver Version - ${SOURCENAME}-${SOURCEVERSION}
xf86-video-nvidia:
E1F


# Add Licence


mkdir -p ${INSTALLDIR}/etc/licences.d

cat <<E2F > ${INSTALLDIR}/etc/licences.d/nvidia-licence
LICENCE NAME: License For Customer Use of NVIDIA Software
LICENCED PACKAGES: nvidia-driver


E2F

cat ${BUILDIR}/${SOURCEDIR}/LICENSE >> ${INSTALLDIR}/etc/licences.d/nvidia-licence


# Add buildscript and patch


mkdir -p ${INSTALLDIR}/usr/src/${SOFTNAME}-${SOURCEVERSION}
( cd ${BUILDIR} ; cp build-${SOFTNAME}.sh ${INSTALLDIR}/usr/src/${SOFTNAME}-${SOURCEVERSION}/ )

# Compress The Package


cd ${INSTALLDIR}
makepkg -l y -c n /tmp/${PACKAGE}.txz



cd ${BUILDIR}
rm -rf {${INSTALLDIR},${SOURCEDIR}}
